﻿/*
 * Box Social™
 * http://boxsocial.net/
  * Copyright © 2007, David Smith
 * 
 * $Id:$
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Security;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using BoxSocial;
using BoxSocial.Internals;
using BoxSocial.Applications.Gallery;

namespace BoxSocial.FrontEnd
{
    public partial class identicon : TPage
    {
        HttpContext httpContext;
        public identicon()
            : base()
        {
            httpContext = HttpContext.Current;
            this.Load += new EventHandler(Page_Load);
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            string profileUserName = core.Http["un"];
            string mode = core.Http["mode"];
            bool retina = core.Http["retina"] == "true";
            User profileOwner;

            int width = 100;

            if (retina)
            {
                switch (mode)
                {
                    case "icon":
                        width = 100;
                        break;
                    case "tile":
                        width = 200;
                        break;
                    case "square":
                    case "high":
                        width = 400;
                        break;
                    case "tiny":
                        width = 160;
                        break;
                    case "thumb":
                        width = 320;
                        break;
                    case "mobile":
                        width = 640;
                        break;
                    case "display":
                        width = 1280;
                        break;
                    case "full":
                    case "ultra":
                        width = 2560;
                        break;
                }
            }
            else
            {
                switch (mode)
                {
                    case "icon":
                        width = 50;
                        break;
                    case "tile":
                        width = 100;
                        break;
                    case "square":
                        width = 200;
                        break;
                    case "high":
                        width = 400;
                        break;
                    case "tiny":
                        width = 80;
                        break;
                    case "thumb":
                        width = 160;
                        break;
                    case "mobile":
                        width = 320;
                        break;
                    case "display":
                        width = 640;
                        break;
                    case "full":
                        width = 1280;
                        break;
                    case "ultra":
                        width = 2560;
                        break;
                }
            }

            try
            {

                profileOwner = new User(core, profileUserName);
            }
            catch
            {
                core.Functions.Generate404();
                return;
            }

            if (profileOwner != null)
            {
                if (profileOwner.UserInfo.DisplayPictureId > 0)
                {
                    httpContext.Response.Redirect(string.Format("/memberpage.aspx?un={0}&path=/images/_{1}/_{0}.png", profileUserName, mode), true);
                    return;
                }
            }

            Response.Cache.SetCacheability(HttpCacheability.Public);
            Response.Cache.SetMaxAge(new TimeSpan(10, 0, 0));
            Response.Cache.SetLastModified(DateTime.Now.Subtract(new TimeSpan(10, 0, 0)));
            Response.ContentType = "image/png";
            Response.Clear();

            byte[] userBytes = System.Text.Encoding.UTF8.GetBytes(profileUserName);
            MD5 md5 = MD5.Create();
            int hash = BitConverter.ToInt32(md5.ComputeHash(userBytes),0);
            Image image = Identicon.CreateIdenticon(hash, width, false);

            string imagePath = string.Empty;
            if (retina)
            {
                imagePath = Path.Combine(Path.Combine(Path.Combine(Path.Combine(Server.MapPath("./"), "images"), "user"), "_" + mode), string.Format("{0}@2x.png",
                    profileUserName));
            }
            else
            {
                imagePath = Path.Combine(Path.Combine(Path.Combine(Path.Combine(Server.MapPath("./"), "images"), "user"), "_" + mode), string.Format("{0}.png",
                    profileUserName));
            }

            try
            {
                FileStream newFileStream = new FileStream(imagePath, FileMode.Create);
                image.Save(newFileStream, ImageFormat.Png);
                newFileStream.Close();
            }
            catch { }

            MemoryStream newStream = new MemoryStream();
            image.Save(newStream, ImageFormat.Png);

            core.Http.WriteStream(newStream);

            if (db != null)
            {
                db.CloseConnection();
            }

            core.Prose.Close();
            //core.Dispose();
            //core = null;

            Response.End();
        }
    }
}
